"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateHeaderRows = exports.getFlattenedColumns = exports.transformColumns = void 0;
const types_1 = require("../types");
const lodash_1 = require("lodash");
/**
 * Transform columns from the table prop to an internal form.
 * Still hierarchical so it can better determine which top level columns to hide/show.
 *
 * @param tableId Table Identifier to help keep some keys unique.
 * @param propColumns The list of columns from the table props.
 * @param parentKey Internal use only for hierarchical data.
 */
function transformColumns(tableId, propColumns, columnVisibility, groupBy, parentKey = '') {
    var _a, _b, _c, _d, _e, _f;
    let columns = [];
    let groupByNames = groupBy.map(g => g.column);
    let index = 0, numColumns = propColumns.length;
    for (; index < numColumns; index++) {
        const column = propColumns[index];
        if (typeof column.accessor === 'undefined' && typeof column.getValue === 'undefined' && typeof column.columns === 'undefined')
            console.error(`Column value is inaccessible and is not a Header Group`);
        if (typeof column.columns !== 'undefined' && (typeof column.accessor !== 'undefined' || typeof column.getValue !== 'undefined'))
            console.warn(`Header Group specified - accessor to value is unnecessary`);
        let key = getColumnKey(tableId, parentKey, index, column);
        let enabled = (0, types_1.resolve)(column.enabled, true);
        let visibleByDefault = (0, types_1.resolve)(column.visibleByDefault, true);
        let isVisible = getColumnIsVisible(key, { enabled, visibleByDefault }, columnVisibility);
        let filter = (0, lodash_1.cloneDeep)(column.filter);
        if (filter && !filter.filterKey) {
            filter.filterKey = (_a = column.name) !== null && _a !== void 0 ? _a : (typeof column.accessor === 'string' ? column.accessor : undefined);
        }
        const columnSearch = (0, lodash_1.cloneDeep)(column.columnSearch);
        if (columnSearch) {
            if (columnSearch.type === 'string') {
                columnSearch.columnSearchOperator = (_b = columnSearch.columnSearchOperator) !== null && _b !== void 0 ? _b : 'con';
            }
            else if (columnSearch.type === 'boolean' || columnSearch.type === 'select') {
                if (columnSearch.type === 'select') {
                    columnSearch.options = ((_c = columnSearch.options) === null || _c === void 0 ? void 0 : _c.length) ? columnSearch.options : [];
                }
                columnSearch.columnSearchOperator = (_d = columnSearch.columnSearchOperator) !== null && _d !== void 0 ? _d : 'eq';
            }
        }
        let transformedColumn = {
            key,
            render: column.render,
            renderGroup: column.renderGroup,
            renderFooter: column.renderFooter,
            accessor: column.accessor,
            getValue: column.getValue,
            className: column.className,
            name: (_e = column.name) !== null && _e !== void 0 ? _e : (typeof column.accessor === 'string' ? column.accessor : undefined),
            filter,
            isPrimaryKey: column.isPrimaryKey,
            editor: column.editor,
            canEdit: column.canEdit,
            EditorWrapper: column.EditorWrapper,
            preMDRColumnWidth: column.preMDRColumnWidth,
            header: (0, types_1.resolve)(column.header, ''),
            fixed: (0, types_1.resolve)(column.fixed, false),
            sortable: (0, types_1.resolve)(column.sortable, true),
            defaultSortDir: (0, types_1.resolve)(column.defaultSortDir, 'asc'),
            enabled,
            visibleByDefault,
            canToggleVisibility: (0, types_1.resolve)(column.canToggleVisibility, true),
            columns: !!column.columns
                ? transformColumns(tableId, column.columns, columnVisibility, groupBy, key)
                : undefined,
            isVisible,
            sortIndex: 0,
            rowSpan: 1,
            colSpan: 1,
            isGrouped: false,
            columnSearch,
        };
        transformedColumn.isGrouped = transformedColumn.name ? !!groupByNames.includes(transformedColumn.name) : false;
        if ((_f = transformedColumn.columns) === null || _f === void 0 ? void 0 : _f.length) {
            let visibleChildren = transformedColumn.columns.filter(c => c.isVisible).length;
            if (visibleChildren <= 1) {
                // add all the children to this level (so non-visible leaf-columns can still appear in selector)
                columns = columns.concat(transformedColumn.columns);
            }
            else {
                for (let col of transformedColumn.columns)
                    col.parent = transformedColumn;
                columns.push(transformedColumn);
            }
        }
        else {
            columns.push(transformedColumn);
        }
    }
    return columns;
}
exports.transformColumns = transformColumns;
function getColumnKey(tableId, parentKey, index, column) {
    let key = '';
    if ((!!column.key || !!column.accessor) && !column.columns) {
        if (typeof column.key !== 'undefined')
            key = column.key;
        else if (typeof column.accessor === 'undefined')
            throw new Error(`Column key must be defined in the absense of a column accessor`);
        else {
            key = (typeof column.accessor === 'number')
                ? column.accessor.toString()
                : column.accessor;
        }
    }
    if (!key) {
        key = `${parentKey}${!!parentKey ? '' : tableId}_c${index}`;
    }
    return key;
}
function getColumnIsVisible(key, column, columnVisibility) {
    if (!column.enabled)
        return false;
    let isVisible = column.visibleByDefault;
    if (typeof columnVisibility[key] !== 'undefined')
        isVisible = columnVisibility[key];
    return isVisible;
}
function getFlattenedColumns(visibleColumns, flattened = []) {
    var _a;
    for (let c of visibleColumns) {
        if ((_a = c.columns) === null || _a === void 0 ? void 0 : _a.length) {
            getFlattenedColumns(c.columns, flattened);
        }
        else {
            flattened.push(c);
        }
    }
    return flattened;
}
exports.getFlattenedColumns = getFlattenedColumns;
function generateHeaderRows(columns, columnOrder) {
    const fixedColumns = columns.filter(c => !!c.fixed);
    const fixedLeftColumns = fixedColumns.filter(c => c.fixed === 'left').map(c => c.key);
    const fixedRightColumns = fixedColumns.filter(c => c.fixed === 'right').map(c => c.key);
    const normalColumns = columns.filter(c => !fixedColumns.includes(c)).map(c => c.key);
    const defaultOrder = [...fixedLeftColumns, ...normalColumns, ...fixedRightColumns];
    for (let i = 0, l = defaultOrder.length; i < l; i++) {
        let column = columns.find(c => c.key === defaultOrder[i]);
        if (column)
            column.sortIndex = i;
    }
    if (columnOrder.length) {
        const sortedOrder = [...fixedLeftColumns, ...columnOrder, ...fixedRightColumns];
        for (let i = 0, l = sortedOrder.length; i < l; i++) {
            let column = columns.find(c => c.key === sortedOrder[i]);
            if (column)
                column.sortIndex = i;
        }
        columns.sort((a, b) => a.sortIndex - b.sortIndex);
    }
    return generateHeaderRow(columns);
}
exports.generateHeaderRows = generateHeaderRows;
function generateHeaderRow(columns, rows = []) {
    let currentRow = [];
    let parentRow = [];
    let lastParent = null;
    let currentColumnSpan = 1;
    for (let column of columns) {
        if (!column.isVisible || column.isGrouped)
            continue;
        currentRow.push(column);
        if (!!lastParent && lastParent === column.parent) {
            currentColumnSpan += column.colSpan;
            lastParent.colSpan = currentColumnSpan;
            if (!parentRow.includes(lastParent)) {
                parentRow.push(lastParent);
            }
        }
        else {
            currentColumnSpan = column.colSpan;
        }
        lastParent = column.parent;
    }
    if (parentRow.length) {
        let actualParentRow = [];
        let actualCurrentRow = [];
        for (let column of columns) {
            if (column.parent && parentRow.includes(column.parent)) {
                if (!actualParentRow.includes(column.parent))
                    actualParentRow.push(column.parent);
                actualCurrentRow.push(column);
            }
            else {
                column.rowSpan++;
                actualParentRow.push(column);
            }
        }
        generateHeaderRow(actualParentRow, rows);
        rows.push(actualCurrentRow);
    }
    else {
        rows.push(currentRow);
    }
    return rows;
}
/*
|       a       |       b       |
|   c   |   d   |   e   |   f   |
| g | h | i | j | k | l | m | n |

{
  a: {
    c: { g, h }
    d: { i, j }
  }
  b: {
    e: { k , l }
    f: { m , n }
  }
}

a
  c, [g,h]
*/ 
