"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertToQS = exports.convertFromQS = exports.transformFilterValue = exports.transformTableFiltersToColumns = void 0;
const types_1 = require("../components/table/types");
const convertValue_1 = require("@borvik/querystring/dist/convert/convertValue");
const helpers_1 = require("../components/filter/helpers");
function transformTableFiltersToColumns(filters) {
    return filters.map(filter => ({
        isVisible: true,
        rowSpan: 1,
        colSpan: 1,
        sortIndex: 0,
        isGrouped: false,
        header: '',
        fixed: false,
        sortable: false,
        defaultSortDir: 'asc',
        enabled: false,
        visibleByDefault: false,
        canToggleVisibility: false,
        key: filter.filterKey + '_filterOnly',
        filter
    }));
}
exports.transformTableFiltersToColumns = transformTableFiltersToColumns;
function transformFilterValue(value, parseType, operator) {
    // based on operator depends on how value should be transformed
    // ex: `==` has a single value
    //     `in` has potentially many values
    //     `between` has two values
    switch (operator) {
        // currently don't have other operators available
        case 'bet':
        case 'nbet':
            let arrBetweenValue = Array.isArray(value)
                ? value
                : (typeof value === 'undefined' || value === null)
                    ? []
                    : [value];
            if (parseType.match(/\[\]$/))
                return (0, convertValue_1.convertValue)(arrBetweenValue, parseType);
            return arrBetweenValue.map(v => (0, convertValue_1.convertValue)(v, parseType));
        case 'any':
        case 'none':
            let arrAnyValue = Array.isArray(value)
                ? value
                : (typeof value === 'undefined' || value === null)
                    ? []
                    : [value];
            if (parseType.match(/\[\]$/))
                return (0, convertValue_1.convertValue)(arrAnyValue, parseType);
            return arrAnyValue.map(v => (0, convertValue_1.convertValue)(v, parseType));
        default: // single value operators
            return (0, convertValue_1.convertValue)(value, parseType);
    }
}
exports.transformFilterValue = transformFilterValue;
/**
 * Handles extracting `filter` from query string to pass to conversions
 */
function convertFromQS(qsFilter, columns) {
    var _a;
    let newFilter = { groupOperator: 'and', filters: [] };
    let actualQSFilter = (_a = qsFilter.filter) !== null && _a !== void 0 ? _a : {};
    let filterKeys = Object.keys(actualQSFilter);
    if (!filterKeys.length)
        return newFilter;
    return convertFilterFromQS(actualQSFilter, columns);
}
exports.convertFromQS = convertFromQS;
/**
 * Handles top-level `filter` conversion
 */
function convertFilterFromQS(qsFilter, columns) {
    let newFilter = { groupOperator: 'and', filters: [] };
    if (Array.isArray(qsFilter)) {
        // likely top-level, filtergroup - process group
        newFilter.filters = convertFilterGroupFromQS(qsFilter, columns);
    }
    else if (typeof qsFilter === 'object') {
        let keys = Object.keys(qsFilter);
        if (!keys.length) {
            console.error('Invalid query string filter object');
            return newFilter;
        }
        if (keys.length === 1 && ['and', 'or'].includes(keys[0])) {
            let operator = keys[0];
            newFilter.groupOperator = operator;
            if (!Array.isArray(qsFilter[operator]) && typeof qsFilter[operator] !== 'object') {
                console.error('Invalid query filter object');
                return newFilter;
            }
            let subFilter = Array.isArray(qsFilter[operator])
                ? qsFilter[operator]
                : [qsFilter[operator]];
            newFilter.filters = convertFilterGroupFromQS(subFilter, columns);
        }
        else {
            newFilter.filters = convertFilterGroupFromQS([qsFilter], columns);
        }
    }
    else {
        console.error('Unable to parse filter from query string (invalid filter)');
    }
    return newFilter;
}
/**
 * Handles recursive `filter` conversion for QueryFilterGroup
 */
function convertFilterGroupFromQS(qsFilter, columns) {
    return qsFilter.reduce((list, filter) => {
        // could be filteritem or filtergroup
        let keys = Object.keys(filter);
        if (!keys.length) {
            console.error('Invalid query string filter object');
            return list;
        }
        if (keys.length === 1 && ['and', 'or'].includes(keys[0])) {
            let operator = keys[0];
            // filtergroup
            let group = { groupOperator: 'and', filters: [] };
            group.groupOperator = operator;
            if (!Array.isArray(filter[operator]) && typeof filter[operator] !== 'object') {
                console.error('Invalid query filter object');
                return list;
            }
            let subFilter = Array.isArray(filter[operator])
                ? filter[operator]
                : [filter[operator]];
            group.filters = convertFilterGroupFromQS(subFilter, columns);
            list.push(group);
            return list;
        }
        let opIndex = keys.indexOf('op');
        if (opIndex >= 0) {
            if (keys.length !== 2 && !(keys.length === 3 && keys.includes('meta'))) {
                console.error('Invalid qs QueryFilterItem');
                return list;
            }
            let colIdx = 1 - opIndex;
            let key = keys[colIdx];
            let item = convertFilterItemFromQS(filter, key, columns);
            if (item) {
                list.push(item);
            }
            return list;
        }
        // probably a simple object structure
        for (let key of keys) {
            let item = convertFilterItemFromQS(filter, key, columns);
            if (item)
                list.push(item);
        }
        return list;
    }, []);
}
/**
 * Handles conversion of QueryFilterItem
 */
function convertFilterItemFromQS(filter, key, columns) {
    var _a, _b, _c, _d, _e;
    // find the column
    let column = columns.find(c => { var _a; return ((_a = c.filter) === null || _a === void 0 ? void 0 : _a.filterKey) === key; });
    if (!column)
        return null;
    let operator = (_b = (_a = filter.op) !== null && _a !== void 0 ? _a : column.filter.defaultOperator) !== null && _b !== void 0 ? _b : (((_c = column.filter.operators) === null || _c === void 0 ? void 0 : _c.length)
        ? column.filter.operators[0]
        : 'eq');
    let value = null;
    let metadata = undefined;
    if (filter.meta) {
        metadata = filter.meta;
    }
    try {
        if (['nul', 'nnul'].includes(operator)) {
            let nullValue = transformFilterValue(filter[key], 'boolean', 'eq');
            operator = !nullValue ? 'nnul' : 'nul';
        }
        else {
            let parseType = (_d = column.filter) === null || _d === void 0 ? void 0 : _d.parseAsType;
            if (!parseType) {
                switch ((_e = column.filter) === null || _e === void 0 ? void 0 : _e.type) {
                    case 'boolean':
                        parseType = 'boolean';
                        break;
                    case 'number':
                        parseType = 'number';
                        break;
                    case 'string':
                    case 'email':
                        parseType = 'string';
                        break;
                    default:
                        parseType = 'any';
                }
            }
            value = transformFilterValue(filter[key], parseType, operator);
        }
    }
    catch (err) {
        console.error(err);
        return null;
    }
    let filterItem = {
        column: key,
        value,
        operator,
        meta: metadata,
    };
    return filterItem;
}
function convertToQS(filters, columns) {
    if (!filters.filters.length)
        return { filter: null };
    let qsFilter = convertFiltersToQS(filters.filters, columns);
    if (filters.groupOperator === 'or')
        qsFilter = { or: qsFilter };
    return { filter: qsFilter };
}
exports.convertToQS = convertToQS;
function convertFiltersToQS(filters, columns) {
    let hasNestedQuery = false, allDefaultOperators = true, columnListedMultiple = false, columnHasMeta = false;
    let simpleColumnCheck = [];
    filters.every(f => {
        if (!(0, types_1.isFilterItem)(f)) {
            hasNestedQuery = true;
            return false; // falsy exits `every`
        }
        else {
            if (f.meta) {
                columnHasMeta = true;
                return false;
            }
            if (simpleColumnCheck.includes(f.column)) {
                columnListedMultiple = true;
                return false;
            }
            simpleColumnCheck.push(f.column);
            let column = columns.find(c => { var _a; return ((_a = c.filter) === null || _a === void 0 ? void 0 : _a.filterKey) === f.column; });
            let defaultOp = (0, helpers_1.getDefaultOperator)(column === null || column === void 0 ? void 0 : column.filter);
            if (f.operator !== defaultOp) {
                allDefaultOperators = false;
                return false; // falsy exits `every`
            }
        }
        return true;
    });
    if (!hasNestedQuery && !columnHasMeta && allDefaultOperators && !columnListedMultiple) {
        let simpleFilter = {};
        for (let f of filters) {
            let ff = f;
            if (ff.operator === 'nul')
                simpleFilter[ff.column] = true;
            else if (ff.operator === 'nnul')
                simpleFilter[ff.column] = false;
            else
                simpleFilter[ff.column] = ff.value;
        }
        return simpleFilter;
    }
    let result = filters.map(f => {
        if (!(0, types_1.isFilterItem)(f)) {
            return { [f.groupOperator]: convertFiltersToQS(f.filters, columns) };
        }
        else {
            let column = columns.find(c => { var _a; return ((_a = c.filter) === null || _a === void 0 ? void 0 : _a.filterKey) === f.column; });
            let defaultOp = (0, helpers_1.getDefaultOperator)(column === null || column === void 0 ? void 0 : column.filter);
            let usedOp = f.operator;
            let filterValue = f.value;
            if (f.operator === 'nul')
                filterValue = true;
            else if (f.operator === 'nnul') {
                filterValue = false;
                usedOp = 'nul';
            }
            if (f.operator === defaultOp) {
                if (f.meta) {
                    return { [f.column]: filterValue, meta: f.meta };
                }
                return { [f.column]: filterValue };
            }
            if (f.meta) {
                return { [f.column]: filterValue, op: usedOp, meta: f.meta };
            }
            return { [f.column]: filterValue, op: usedOp };
        }
    });
    return result;
}
